//=================================================================================================
/// \file:		dcf77.h
///
/// \date:		23.04.2014
///
/// \version: 1.0
///
/// \author:  Daniel 
//=================================================================================================
#ifndef DCF77_AUSWERTUNG_H_
#define DCF77_AUSWERTUNG_H_
//-------------------------------------------------------------------------------------------------
// Includes
//-------------------------------------------------------------------------------------------------
#include <avr/io.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>
// ACHTUNG: Die Taktfrequenz darf 8 MHz nicht berschreiten, da der 8-Bit Timer sonst zu klein ist
#define F_CPU 																					8000000
#include <util/delay.h>
#include <avr/interrupt.h>
//-------------------------------------------------------------------------------------------------
// Makros
//-------------------------------------------------------------------------------------------------
// PINKONFIGURATION / HARDWARE
#define DCF77_SIGNAL_PORT																PORTB
#define DCF77_SIGNAL_PIN																PINB
#define DCF77_SIGNAL_PIN_NUMBER													PINB0
#define DCF77_MODULE_POWER_DDR													DDRB
#define DCF77_MODULE_POWER_PORT													PORTB
#define DCF77_MODULE_POWER_PIN													PINB
#define DCF77_MODULE_POWER_PIN_NUMBER										PINB1
#define DCF77_CHECKING_LED_DDR													DDRB
#define DCF77_CHECKING_LED_PORT													PORTB
#define DCF77_CHECKING_LED_PIN													PINB
#define DCF77_CHECKING_LED_PIN_NUMBER										PINB2
// Defines zum Ein- und Ausschalten des DCF77-Empfmgermoduls
#define SWITCH_ON																				1
#define SWITCH_OFF																			0

// TIMERKONFIGURATION
#define PRESCALER_TIMER_1																64
#define INTERRUPTS_PER_SECOND														100
// Die 1 wird abgezogen, weil der Timer bei 0 anfngt zu zhlen und nicht bei 1
#define COMPARE_CAPTURE_VALUE														(((F_CPU/PRESCALER_TIMER_1)/INTERRUPTS_PER_SECOND)-1)

// SIGNALABTASTUNG UND -AUSWERTUNG
#define HIGH_LEVEL																			1
#define LOW_LEVEL																				0
// Anzahl der Bits des DCF77-Signals
#define NUMBER_OF_BITS																	59
// Anzahl der Flanken, bis das Empngermodul als betriebsbereit gilt
#define NUMBER_OF_EDGES_UNTIL_MODULE_IS_READY						3
// Grenzwerte fr die Einteilung der Bits in 0 und 1
#define MAXIMUM_NUMBER_OF_PULSES_FOR_STARTMARK					3
#define MINIMUM_NUMBER_OF_PULSES_FOR_LOGICAL_0					7
#define MAXIMUM_NUMBER_OF_PULSES_FOR_LOGICAL_0					13
#define MINIMUM_NUMBER_OF_PULSES_FOR_LOGICAL_1					17
#define MAXIMUM_NUMBER_OF_PULSES_FOR_LOGICAL_1					23
// Wertigkeit einzelner Bits bei der Berechnung der Zeit und des Datums
#define BIT_SIGNIFICANCE_1															1
#define BIT_SIGNIFICANCE_2															2
#define BIT_SIGNIFICANCE_4															4
#define BIT_SIGNIFICANCE_8															8
#define BIT_SIGNIFICANCE_10															10
#define BIT_SIGNIFICANCE_20															20
#define BIT_SIGNIFICANCE_40															40
// Grundwert der Jahreszahl des Signals, auf dem die empfangenen Jahre addiert werden
#define BASE_YEAR																				2000
// Defines fr die Parittsbit- und Plausibilitts-berprfung
#define MODULO_TWO																			2
#define TRANSMISSION_ERROR															0
#define	PLAUSIBILITY_ERROR															0	
#define UPDATE_ERROR																		0
#define TRANSMISSION_OK																	1
#define PLAUSIBILITY_OK																	1
#define UPDATE_OK																				1
#define PARITY_BIT_MINUTES															28
#define PARITY_BIT_HOURES																35
#define PARITY_BIT_DATE																	58
#define FIRST_BIT_MINUTE																21
#define LAST_BIT_MINUTE																	27
#define FIRST_BIT_HOUR																	29
#define LAST_BIT_HOUR																		34
#define FIRST_BIT_DATE																	36
#define LAST_BIT_DATE																		57

// STATUSMELDUNGEN
// Makros fr Statusmeldungen
#define WAIT_FOR_MODULE																	1
#define MODULE_IS_READY_SEARCH_SIGNAL										2
#define MODULE_IS_READY_SIGNAL_FOUND_TIME_IS_UPDATING		3
// Makros fr die Positionen der Statusmeldungen
#define FIRST_STATUS_MESSAGE_PART_A_LINE								2
#define FIRST_STATUS_MESSAGE_PART_A_COLUMN							2
#define FIRST_STATUS_MESSAGE_PART_B_LINE								2
#define FIRST_STATUS_MESSAGE_PART_B_COLUMN							6
#define SECOND_STATUS_MESSAGE_LINE											4
#define SECOND_STATUS_MESSAGE_COLUMN										5
#define THIRD_STATUS_MESSAGE_LINE												6
#define THIRD_STATUS_MESSAGE_COLUMN											8
// Makros fr die Positionen des Cursors
#define CURSOR_POSITION_FIRST_STATUS_MESSAGE_LINE				2
#define CURSOR_POSITION_FIRST_STATUS_MESSAGE_COLUMN			39
#define CURSOR_POSITION_SECOND_STATUS_MESSAGE_LINE			4
#define CURSOR_POSITION_SECOND_STATUS_MESSAGE_COLUMN		36
#define CURSOR_POSITION_THIRD_STATUS_MESSAGE_LINE				6
#define CURSOR_POSITION_THIRD_STATUS_MESSAGE_COLUMN			33
#define CURSOR_PARK_POSITION_LINE												1
#define CURSOR_PARK_POSITION_COLUMN											45

// ZEIT- UND DATUMSWERK
// Defines fr Zeit- und Datumswerte
#define LAST_SECOND_OF_MINUTE														59
#define LAST_MINUTE_OF_HOUR															59
#define LAST_HOUR_OF_DAY																23
#define LATEST_MONTH_DAY_IN_YEAR												31
#define LAST_DAY_OF_MONTH_28														28
#define LAST_DAY_OF_MONTH_29														29
#define LAST_DAY_OF_MONTH_30														30
#define LAST_DAY_OF_MONTH_31														31
// Wochentage
#define MONDAY																					1
#define TUESDAY																					2
#define WEDNESDAY																				3
#define THURSDAY																				4
#define FRIDAY																					5
#define SATURDAY																				6
#define SUNDAY																					7
// Monate
#define JANUARY																					1
#define FEBRUARY																				2
#define MARCH																						3
#define APRIL																						4
#define MAY																							5
#define JUNE																						6
#define JULY																						7
#define AUGUST																					8
#define SEPTEMBER																				9
#define OCTOBER																					10
#define NOVEMBER																				11
#define DECEMBER																				12	
//-------------------------------------------------------------------------------------------------
// Variables
//-------------------------------------------------------------------------------------------------
// ZEIT- UND DATUMSWERK
// Sturuktur definieren
struct timeAndDate
{
	uint8_t second;
	uint8_t minute;
	uint8_t hour;
	uint8_t weekDay;
	uint8_t monthDay;
	uint8_t month;
	uint16_t year;
};
// Struct fr Systemzeit initialisieren, dieser speichert die Daten, welche genutzt werden
volatile struct timeAndDate systemTime;
// Struct fr Zeitkopie initialisieren, dieser speichert die Daten, welche Zwischenwerte darstellen
volatile struct timeAndDate copyTime;

// SIGNALABTASTUNG
// Variable zhlt die Interrupts des Timers 2
volatile uint8_t timer1InterruptCounter;
// Variable speichert den Bitstrom des DCF77-Signals
volatile bool dcf77SignalBuffer[NUMBER_OF_BITS];
// Variable speichert die Stelle im Bitstrom-Puffer, an der das nchste Bit gespeichert wird
volatile uint8_t dcf77BitCounter;
// Variable speichert den zuletzt detektierten Signalpegel (dient zur Steuerung der Signal-LED)
// Auf High gesetzt, damit mindestens ein LOW-Pegel anliegen muss, bevor eine steigende Flanke erfasst wird
volatile bool dcf77LastSignalLevel;
// Variable speichert die Anzahl der ersten positiven Flanken des Empfngermoduls nach dessen Aktivierung
volatile uint8_t dcf77FirstEdgesCounter;
// Variable signalisiert den Beginn einer neuen Zeitbertragung (wir beim 59. Sekundenbit gesetzt)
volatile bool dcf77SignalStartFound;
// Variable wird einmalig bei Beginn der Signalabtastung gesetzt und setzt den Interruptcounter auf 0
volatile bool dcf77ResetInterruptCounter;
// Variable zhlt die High-Phasen (10 ms-Abschnitte) innerhalb einer Sekunde
volatile uint8_t dcf77HighPulseCounter;

// DATENAUSGABE
// Variable wird jede Sekunde in der ISR gelscht
// und im Hauptprogramm nach einer AUsgabe auf dem LCD wieder gesetzt
volatile bool timeIsSetet;
// Variable steuert die Ausgabe der Statusmeldungen
// Jede Statusmeldung wird nur einmal ausgegeben
volatile uint8_t dcf77CurrentMessage;
//-------------------------------------------------------------------------------------------------
// Prototypes
//-------------------------------------------------------------------------------------------------
// Funktion initialisiert den Puffer fr den Bitstrom des DCF77-Signals
void Dcf77InitSignalBuffer(void);
// Funktion initialisiert den Timer 1
void Dcf77Timer1Init(void);
// Funktion schaltet das Empfngermodul ein und aus
void Dcf77SwitchDcfModule(bool);
// Funktion initialisiert smtliche Ein- und Ausgnge fr die Signalerfassung, etc.
void Dcf77Init(void);
// Funktion errechnet Zeit und Datum aus dem empfangenen Bitstrom
void Dcf77CalculateTimeAndDate(void);
// Funktion berprft die bertragung (Parittsbit-Vergleich)
bool Dcf77CheckTransmission(void);
// Funktion berprft die Plausibilitt der errechneten Zeit und des Datums
bool Dcf77CheckplausibilityOfData(void);
// Funktion gibt Infomeldungen ber den Zustand der Zeitaktualisierung ber ein LCD aus
void Dcf77WriteMessage(uint8_t status);

#endif 