//=================================================================================================
/// \file:		Graphics.h
///
/// \date:		17.07.2014
///
/// \version: 1.1
///
/// \author:  Daniel 
//=================================================================================================
#ifndef GRAPHICS_H_
#define GRAPHICS_H_
//-------------------------------------------------------------------------------------------------
// Includes
//-------------------------------------------------------------------------------------------------
#include <avr/io.h>
#include <stdint.h>
#define F_CPU 													8000000
#include <util/delay.h>
#include <stdbool.h>
#include <string.h>
//-------------------------------------------------------------------------------------------------
// Makros
//-------------------------------------------------------------------------------------------------
// Gre der Bitmaps
#define SMILEY_SIZE_IN_PIXELS						32
// Ausrichtung der Segmente der 7-Segment-Anzeige
#define HORIZONTAL											0
#define VERTICAL												1
// Positionsoffset einzelner Segmente bezogen aud der Position innerhalb der Zahl
// Offset kleine Zahl
#define OFFSET_SMALL_SEGMENT_A_X				3
#define OFFSET_SMALL_SEGMENT_A_Y				2
#define OFFSET_SMALL_SEGMENT_B_X				18
#define OFFSET_SMALL_SEGMENT_B_Y				4
#define OFFSET_SMALL_SEGMENT_C_X				18
#define OFFSET_SMALL_SEGMENT_C_Y				19
#define OFFSET_SMALL_SEGMENT_D_X				3
#define OFFSET_SMALL_SEGMENT_D_Y				32
#define OFFSET_SMALL_SEGMENT_E_X				0
#define OFFSET_SMALL_SEGMENT_E_Y				19
#define OFFSET_SMALL_SEGMENT_F_X				0
#define OFFSET_SMALL_SEGMENT_F_Y				4
#define OFFSET_SMALL_SEGMENT_G_X				3
#define OFFSET_SMALL_SEGMENT_G_Y				17
// Offset groe Zahl
#define OFFSET_LARGE_SEGMENT_A_X				4
#define OFFSET_LARGE_SEGMENT_A_Y				0
#define OFFSET_LARGE_SEGMENT_B_X				30
#define OFFSET_LARGE_SEGMENT_B_Y				3
#define OFFSET_LARGE_SEGMENT_C_X				30
#define OFFSET_LARGE_SEGMENT_C_Y				31
#define OFFSET_LARGE_SEGMENT_D_X				4
#define OFFSET_LARGE_SEGMENT_D_Y				56
#define OFFSET_LARGE_SEGMENT_E_X				0
#define OFFSET_LARGE_SEGMENT_E_Y				31
#define OFFSET_LARGE_SEGMENT_F_X				0
#define OFFSET_LARGE_SEGMENT_F_Y				3
#define OFFSET_LARGE_SEGMENT_G_X				4
#define OFFSET_LARGE_SEGMENT_G_Y				28
// Positionasoffsets fr die Ausgabe einzelner Segmente/ Segmentteile
// Defines gelten fr beide Segmentgren
#define SEGMENT_OFFSET_OF_1							1
#define SEGMENT_OFFSET_OF_2							2
#define SEGMENT_OFFSET_OF_3							3
#define SEGMENT_OFFSET_OF_4							4
#define SEGMENT_OFFSET_OF_5							5
#define SEGMENT_OFFSET_OF_12						12
#define SEGMENT_OFFSET_OF_13						13
#define SEGMENT_OFFSET_OF_14						14
#define SEGMENT_OFFSET_OF_24						24
#define SEGMENT_OFFSET_OF_25						25
#define SEGMENT_OFFSET_OF_26						26
// Positionsangaben fr die Ausgabe der Zahlen auf dem LCD
// Stunden
#define HOUR_FIRST_NUMBER_POSITION_X		1
#define HOUR_FIRST_NUMBER_POSITION_Y		1
#define HOUR_SECOND_NUMBER_POSITION_X		43
#define HOUR_SECOND_NUMBER_POSITION_Y		1
// Doppelpunkt
// Oberer Punkt
#define UPPER_POINT_POSITION_X1					84
#define UPPER_POINT_POSITION_X2					90
#define UPPER_POINT_POSITION_Y1					16
#define UPPER_POINT_POSITION_Y2					22
// Unterer Punkt
#define LOWER_POINT_POSITION_X1					84
#define LOWER_POINT_POSITION_X2					90
#define LOWER_POINT_POSITION_Y1					39
#define LOWER_POINT_POSITION_Y2					45
// Minuten
#define MINUTE_FIRST_NUMBER_POSITION_X	97
#define MINUTE_FIRST_NUMBER_POSITION_Y	1
#define MINUTE_SECOND_NUMBER_POSITION_X	139
#define MINUTE_SECOND_NUMBER_POSITION_Y	1
// Sekunden
#define SECOND_FIRST_NUMBER_POSITION_X	187
#define SECOND_FIRST_NUMBER_POSITION_Y	27
#define SECOND_SECOND_NUMBER_POSITION_X	211
#define SECOND_SECOND_NUMBER_POSITION_Y	27
// Wochentag
#define WEEK_DAY_POSITION_LINE					1
#define WEEK_DAY_POSITION_COLUMN				31
// Datum
// Trennpunkte
#define DATE_POINTS_POSITION_LINE				2
#define DATE_POINTS_POSITION_COLUMN			31
// Tag
#define DATE_DAY_POSITION_LINE					2
#define DATE_DAY_POSITION_COLUMN				31
// Monat
#define DATE_MONTH_POSITION_LINE				2
#define DATE_MONTH_POSITION_COLUMN			34
// Jahr
#define DATE_YEAR_POSITION_LINE					2
#define DATE_YEAR_POSITION_COLUMN				37
// Temperatur
#define TEMPERATURE_PREPOINT						2
#define TEMPERATURE_AFTERPOINT					1
#define TEMPERATURE_POSITION_LINE				3
#define TEMPERATURE_POSITION_COLUMN			31
#define DEGREE_SIGN_POSITION_LINE				3
#define DEGREE_SIGN_POSITION_COLUMN			36
#define UNIT_POSITION_LINE							3
#define UNIT_POSITION_COLUMN						37
//-------------------------------------------------------------------------------------------------
// Variables
//-------------------------------------------------------------------------------------------------
// Bitmaps Smileys
volatile bool happySmiley [SMILEY_SIZE_IN_PIXELS][SMILEY_SIZE_IN_PIXELS];
volatile bool sadSmiley [SMILEY_SIZE_IN_PIXELS][SMILEY_SIZE_IN_PIXELS];
//-------------------------------------------------------------------------------------------------
// Prototypes
//-------------------------------------------------------------------------------------------------
// Funktion zeichnet einen lachenden Smiley 													
void GraphicDrawHappySmiley(uint16_t x, 
														uint16_t y,
														bool color);
// Funktion zeichnet einen traurigen Smiley											
void GraphicDrawSadSmiley(uint16_t x, 
													uint16_t y,
													bool color);
// Funktion zeichnet ein kleines Segment einer 7-Segment-Zahl
void GraphicDrawSmallSegment(uint16_t x,
														 uint16_t y,
														 bool direction,
														 bool color);
// Funktion zeichnet ein groes Segment einer 7-Segment-Zahl
void GraphicDrawLargeSegment(uint16_t x,
														 uint16_t y,
														 bool direction,
														 bool color);														 
// Funktion zeichnet eine kleine 7-Segment-Zahl
void GraphicDrawSmallNumber(uint16_t x,
														uint16_t y,
														uint8_t value);
// Funktion zeichnet eine groe 7-Segment-Zahl
void GraphicDrawLargeNumber(uint16_t x,
														uint16_t y,
														uint8_t value);												
// Funktion zeichnet die Uhrzeit und Datum auf das LCD 
void GraphicWriteDataOnLcd(double temperature);

#endif 