//=================================================================================================
/// \file:		LCD240X064.h
///
/// \date:		16.07.2014
///
/// \version: 1.0
///
/// \author:  Daniel 
//=================================================================================================
#ifndef LCD240X064_H_
#define LCD240X064_H_
//-------------------------------------------------------------------------------------------------
// Includes
//-------------------------------------------------------------------------------------------------
#include <avr/io.h>
#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#define F_CPU 													8000000
#include <util/delay.h>
//-------------------------------------------------------------------------------------------------
// Macros
//-------------------------------------------------------------------------------------------------
// HARDWARE
#define DATA_DDR												DDRD
#define DATA_PORT												PORTD
#define DATA_PIN												PIND
#define CONTROL_DDR											DDRA
#define CONTROL_PORT										PORTA
#define WRITE														PINA0
#define READ														PINA1
#define ENABLE													PINA2
#define COMMAND_DATA										PINA3
#define RESET														PINA4
#define CHARACTER_SIZE									PINA5

// WARTEZEITEN
#define RESET_TIME_US										10
#define ENABLE_TIME_US									2

// DISPLAYMAE/ AUFTEILUNG
#define PIXELS_OF_DISPLAY_X							240
#define PIXELS_OF_DISPLAY_Y							64
// Breite einses Zeichens (6 oder 8 Pixel, wird ber die Fontsize-Leitung festgelegt)
// Die Zeichenbreite hat auch Einfluss auf die Aufteilung des Grafikspeichers
// Ist die Zeichenbreite 8 Pixel, so werden pro Pixelreihe in y-Richtung 30 Bytes Grafikspeicher bentigt
// Ist die Zeichenbreite 6 Pixel, so werden pro Pixelreihe in y-Richtung 40 Bytes Grafikspeicher bentigt
// Bei einer Zeichenbreite von 6 Pixeln, werden die ersten beiden hchstwertigen Bits (7 und 6) nicht benutzt
#define FONT_SIZE												6
// Hhe eines Zeichens
#define FONT_HEIGHT											8
// Zeilen- und Spaltenanzahl des Displays festlegen
#define TEXTLINES_OF_DISPLAY						(PIXELS_OF_DISPLAY_Y/FONT_HEIGHT)
#define COLUMNS_OF_DISLPLAY							(PIXELS_OF_DISPLAY_X/FONT_SIZE)

// SPEICHERVERWALTUNG
// Sicherheitsabstand zwischen Text- und Grafikspeicher
// Dient zum Schutz vor evt. berlappung, ist jedoch nicht unbedingt notwendig
#define SAFETY_OFFSET										5
// Startadresse Textspeicher
#define HOME_ADRESS_TEXT								0x00
// Gre des Textspeicherbereiches
#define TEXT_AREA												(COLUMNS_OF_DISLPLAY*TEXTLINES_OF_DISPLAY)
// Startadresse Grafikspeicher
#define HOME_ADRESS_GRAPHIC							(TEXT_AREA+SAFETY_OFFSET)
// Gre des Grafikspeicherbereiches (Spalten x horizintale Pixellinien)
#define GRAPHIC_AREA										(COLUMNS_OF_DISLPLAY*PIXELS_OF_DISPLAY_Y)
	
// BEFEHLSCODES UND WEITERE CODIERUNGEN
// Allgemein
#define BITSIZE_OF_ONE_BYTE							0x08
#define INITIAL_VALUE										0x00
#define SET_ALL_BITS										0xFF
#define DELETE_ALL_BITS									0x00
#define SET_BIT													0xF8
#define DELETE_BIT											0xF0
#define SET_CURSOR											0x21
#define SET_ADDRESS_POINTER							0x24
#define SET_TEXT_HOME_ADDRESS						0x40
#define SET_TEXT_AREA										0x41
#define SET_GRAPHIC_HOME_ADDRESS				0x42
#define SET_GRAPHIC_AREA								0x43
#define OR_MODE													0x80
#define EXOR_MODE												0x81
#define AND_MODE												0x83
#define LCD_CURSOR_TEXT_GRAPHIC_ON			0x9E
#define LCD_CURSORBLINK_TEXT_GRAPHIC_ON	0x9F
#define CURSOR_PATERN_1									0xA0
#define CURSOR_PATERN_8									0xA7
// Offset der internen Zeichencodierung bezogen auf dei ASCII-Codierung
#define ASCII_OFFSET										32
// ASCII-Code fr Null	
#define ASCII_CODE_FOR_ZERO							0x30
// Codes fr Umlaute
#define CODE_AE_SMALL										0x64
#define CODE_AE_LARGE										0x6E
#define CODE_OE_SMALL										0x74
#define CODE_OE_LARGE										0x79
#define CODE_UE_SMALL										0x61
#define CODE_UE_LARGE										0x7A
// Pixelfarbe			
#define BLUE														0
#define WHITE														1
// Dezimale Ausgabe eine Zahl
#define SIGN														1
#define NO_SIGN													0	
// Fliekommazahl Ausgabe
#define MAX_SIZE_OF_AFTERPOINT					4		
#define OFFSET_FOR_CORRECTION						0.00001	
// Binre Ausgabe einer Zahl
#define SIZE_8_BIT											8
#define SIZE_16_BIT											16
// Allgemeine Defines zum Ein- und Ausschalten
#define SWITCH_ON												1
#define SWITCH_OFF											0

// BENUTZERDEFINIERTE ZEICHEN
// Hhe des Zeichens in Pixeln
#define HEIGHT_SPECIFIC_SIGN						8
// Sonderzeichen
// Gradzeichen
#define DEGREE_SIGN											0xF8
//-------------------------------------------------------------------------------------------------
// Variables
//-------------------------------------------------------------------------------------------------
// Variable zur Speicherung des dezimalen Wertes einer komplett gesetzten Bitreihe
// in Abhngigkeit von der Zeichngre
uint8_t setAllBitsOfFont;
// Bitmaps benutzerdefimierter Zeichen
// Grad-Zeichen
uint8_t bitmapDegreeSign[HEIGHT_SPECIFIC_SIGN];
//-------------------------------------------------------------------------------------------------
// Prototypes
//-------------------------------------------------------------------------------------------------
// Funktion erzeugt einen Enable-Impuls fr die Displaykommunikation
void LcdEnablePulse(void);
// Funktion erzeugt einen Reset-Impuls fr die Displaykommunikation
void LcdReset(void);
// Funktion fragt zwei Statusbits des Displays ab
void LcdCheckStatus(void);
// Funktion sendet ein Kommandobefehl an das Display
void LcdSendCommand(unsigned char command);
// Funktion sendet ein Datenbyte an das Display
void LcdSendData(unsigned char data);
// Funktion sendet Daten an das Display
void LcdWriteData(unsigned char data);
// Funktion lscht alle Speicher (Text und Grafik) des Displays
void LcdClear(void);
// Funktion lscht den gesamten Textspeicher des Displays
void LcdClearText(void);
// Funktion lscht den Textspeicher teilweise
void LcdClearTextPartically(uint8_t line, 
														uint8_t startColumn, 
														uint8_t endColumn);
// Funktion lscht den gesamten Graphikspeicher des Displays
void LcdClearGraphic(void);
// Funktion initialisiert das Display
void LcdInit(void);
// Funktion setzt den Adresspointer an eine Adresse (RAM-Adresse)
void LcdSetAddressPointer(uint16_t address);
// Funktion setzt den Adresspointer an eine Adresse (Spalte und Zeile)
void LcdJumpToPosition(uint8_t line, 
											 uint8_t column);
// Funktion schaltet den Cursor ein oder aus
void LcdSwitchCursor(bool status);
// Funktion setzt den Cursor in eine Reihen- und Spaltenposition
void LcdSetCursor(uint8_t line,
									uint8_t column);
// Funktion schreibt ein Zeichen
void LcdWriteChar(unsigned char character);
// Funktion schreibt ein Zeichenstring an eine bergebene Position 
void LcdWriteString(char string[], 
										uint8_t line, 
										uint8_t column);
// Funktion schreibt eine dezimale Zahl an eine bergebene Position 
void LcdWriteInteger(int16_t value, 
										 uint8_t size,
										 bool statusOfSign, 
										 uint8_t line, 
										 uint8_t column);
// Funktion schreibt eine Fliekommazahl an eine bergebene Position
void LcdWriteDouble(double number,
										uint8_t sizePrePoint,
										uint8_t sizeAfterPoint,
										uint8_t line,
										uint8_t column);
// Funktion gibt eine Zahl binr an eine bergebene Position auf dem LCD aus									 
void LcdWriteBinaryNumber(uint16_t value,
													uint8_t size,
													uint8_t line,
													uint8_t column);											
// Funktion setzt ein einzelnes Pixel an eine bergebene Position
void LcdDrawPixel(uint16_t x, 
									uint16_t y, 
									bool color);
// Funktion zeichnet eine Linie an eine bergebene Position								
void LcdDrawLine(uint8_t x1,
								 uint8_t y1,
								 uint8_t x2,
								 uint8_t y2,
								 bool color);
// Funktion zeichnet ein Rechteckrahmen an eine bergebene Position									
void LcdDrawRectangle(uint16_t x1, 
											uint16_t y1,
											uint16_t x2, 
											uint16_t y2, 
											uint16_t thickness, 
											bool color);
// Funktion zeichnet eine rechteckige Flche an eine bergebene Position
void LcdSetGraphicArea(uint8_t x1,
											 uint8_t y1,
											 uint8_t x2,
											 uint8_t y2,
											 bool color);
// Funktion zeichnet einen Kreis an eine bergebene Position
void LcdDrawCircle(uint8_t xCenter, 
									 uint8_t yCenter, 
									 uint8_t radius,
									 bool color);
// Funktion gibt ein benutzerdefiniertes Zeichen aus
void lcdWriteUseSpecificSign(uint8_t sign,
														 uint8_t line,
														 uint8_t column);
#endif