//=================================================================================================
/// \file:		Temperatursensor.h
///
/// \date:		01.08.2014
///
/// \version: 1.0
///
/// \author:  Daniel 
//=================================================================================================
#ifndef TEMPERATURESENSOR_H_
#define TEMPERATURESENSOR_H_
//-------------------------------------------------------------------------------------------------
// Includes
//-------------------------------------------------------------------------------------------------
#include <avr/io.h>
#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#define F_CPU 																					8000000
#include <util/delay.h>
//-------------------------------------------------------------------------------------------------
// Makros
//-------------------------------------------------------------------------------------------------
// Busadresse des Sensors
#define SLAVE_ADDRESS_MCP9808														0x18
// Zielregister des Sensors
#define REGISTER_CONFIGURATION													0x01
#define REGISTER_UPPER_TEMPERATURE_ALARM_LIMIT					0x02
#define REGISTER_LOWER_TEMPERATURE_ALARM_LIMIT					0x03
#define REGISTER_CRITICAL_TEMPERATURE_LIMIT							0x04
#define REGISTER_AMBIENT_TEMPERATURE										0x05
#define REGISTER_MANUFACTURER_ID												0x06
#define REGISTER_DEVICE_ID															0x07
#define REGISTER_TEMPERATURE_RESOLUTION									0x08
// Temperaturauflsungen des Sensors
#define RESOLUTION_0_POINT_5_C													0x00
#define RESOLUTION_0_POINT_25_C													0x01
#define RESOLUTION_0_POINT_125_C												0x02
#define RESOLUTION_0_POINT_0625_C												0x03
// Temperaturgrenzen
#define MINIMUM_TEMPERATURE															-40
#define MAXIMUM_TEMPERATURE															125
//-------------------------------------------------------------------------------------------------
// Variables
//-------------------------------------------------------------------------------------------------
// Temperaturdifferenz durch Eigenerwrmung des Sensor
volatile double temperaturOfSelfHeating;
//-------------------------------------------------------------------------------------------------
// Prototypes
//-------------------------------------------------------------------------------------------------
// Funktion setzt den Registerpointer des Sensors auf ein bestimmtes Register
void MCP9808SetRegisterPointer(uint8_t registerNumber);
// Funktion verndert die Auflsung des Temperatursensors
void MCP9808SetResolutionRegister(uint8_t resolution);
// Funktion setzt das Konfigurationsregister
void MCP9808SetConfigurationRegister(void);
// Funktion setzt die obere, untere oder kritische Temperaturgrenze, bei der der Alarm ausgelst wird
void MCP9808SetTemperatureLimitAlert(double temperature,
																				uint8_t temperatureRegister);
// Funktion liest das Temperaturregister fr die obere, untere oder kritische Temperaturgrenze, bei der der Alarm ausgelst wird
double MCP9808ReadTemperatureLimitAlert(uint8_t temperatureRegister);
// Funktion initialisiert den Temperatursensor
void MCP9808Init(void);
// Funktion liest die aktuelle Umgebungstemperatur aus
double MCP9808ReadAmbientTemperature(void);
// Funktion liest die Herstellernummer aus
uint16_t MCP9808ReadManufacturerID(void);
// Funktion liest die Bauteilnummer aus
uint8_t MCP9808ReadDeviceID(void);
// Funktion liest die Revisionsnummer aus
uint8_t MCP9808ReadDeviceRevision(void);
// Funktion liest das Auflsungsregister aus
uint8_t MCP9808ReadResolutionRegister(void);
// Gibt die Umgebungstemperatur auf ein LCD aus
void MCP9808WriteAmbientTemperatureOnLcd(double temperature,
																				 uint8_t line,
																				 uint8_t column);

#endif