//=================================================================================================
/// \file:		TwoWireInterface.h
///
/// \date:		26.03.2014
///
/// \version: 1.0
///
/// \author:  Daniel 
//=================================================================================================
#ifndef TWOWIREINTERFACE_H_
#define TWOWIREINTERFACE_H_
//-------------------------------------------------------------------------------------------------
// Includes
//-------------------------------------------------------------------------------------------------
#include <avr/io.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <avr/interrupt.h>
#define F_CPU 																		8000000
#include <util/delay.h>
//-------------------------------------------------------------------------------------------------
// Makros
//-------------------------------------------------------------------------------------------------
// Bitrate
#define BITRATE_HZ																25000
// Slave-Adresse
#define SLAVE_ADDRESS															0x18
// Statusregister-Codes
#define START_CONDITION_SENDED										0x08
#define REPEATED_START_CONDITION_SENDED						0x10
// Master-Transmitter-Mode
#define SLAVE_ADDRESS_WRITE_SENDED_ACK_RECEIVED		0x18
#define DATA_BYTE_SENDED_ACK_RECEIVED							0x28
// Master-Receiver-Mode
#define SLAVE_ADDRESS_READ_SENDED_ACK_RECEIVED		0x40
#define DATA_BYTE_RECEIVED_NACK_SENDED						0x58
#define DATA_BYTE_RECEIVED_ACK_SENDED							0x50
// Operationsarten Lesen/ Schreiben
#define WRITE_DATA																0
#define READ_DATA																	1
// Timeout Zeit
// Der Wert/100 entspricht der Timeout-Zeit in Millisekunden
#define TIME_OUT																	5000
// Fehlerarten 
#define ERROR_NONE																0
#define ERROR_START_CONDITION											1
#define ERROR_REPEATED_START_CONDITION						2
#define ERROR_SLAVE_ADDRESS_WRITE									3
#define ERROR_SLAVE_ADDRESS_READ									4
#define ERROR_DATA_WRITE													5
#define ERROR_DATA_READ														6
// Fehler Ausgabe
#define ERROR_OUTPUT_FIELD_LINE										7
#define ERROR_OUTPUT_FIELD_COLUMN									1
#define ERROR_OUTPUT_LINE													7
#define ERROR_OUTPUT_COLUMN												8
// Anzahl  gesendeter/ empfangener Datenbytes
#define TWO_DATA_BYTES														2	
#define THREE_DATA_BYTES													3	
#define FOUR_DATA_BYTES														4	
#define FIVE_DATA_BYTES														5
//-------------------------------------------------------------------------------------------------
// Variables
//-------------------------------------------------------------------------------------------------
// Fehlerspeicher
volatile uint8_t firstError;
//-------------------------------------------------------------------------------------------------
// Prototypes
//-------------------------------------------------------------------------------------------------
// Funktion initialisiert das TWI als master (setzt die Taktfrequenz)
void TwiInit(uint16_t sclFrequencykHz);
// Funktion erzeugt ein Start-Signal
void TwiStartCondition(void);
// Funktion erzeugt ein wiederholtes Startsignal
void TwiRepeatedStartCondition(void);
// Funktion erzeugt ein Stop-Signal
void TwiStopCondition(void);
// Funktion wartet auf das Setzen der Interrupt-Flag
void TwiWaitForInterruptFlag(void);	
// Funktion sendet das erste Byte eines Datentransfers (Slaveadresse und Operationsbefehl (R/W))
void TwiAddressSlave(uint8_t slaveAddress,
										 bool operation);	
// Funktion sendet ein einzelndes Nutzdaten-Byte an einen Slave
void TwiTransmitSingleByte(uint8_t slaveAddress,
													 uint8_t dataByte);												 
// Funktion empfngt ein einzelnes Nutzdaten-Byte von einem Slave
uint8_t TwiReceiveSingleByte(uint8_t slaveAddress);												 
// Funktion sendet mehrere Nutzdaten-Bytes an einen Slave
void TwiTransmitByteArray(uint8_t slaveAddress,
													uint8_t *firstElementOfDataByteArray,
													uint8_t numberOfDataBytes);
// Funktion empfngt mehrere Nutzdaten-Bytes von einem Slave
void TwiReceiveByteArray(uint8_t slaveAddress,
												 uint8_t *firstElementOfDataByteArray,
												 uint8_t numberOfDataBytes);
// Gibt einen bertragungsfehler auf dem LCD aus
void TwiFailureHandling(void);

#endif

