//=================================================================================================
/// \file:		main.c
///
/// \date:		06.07.2014
///
/// \version: 1.0
///
/// \authors: Daniel 
//=================================================================================================
//-------------------------------------------------------------------------------------------------
//Includes
//-------------------------------------------------------------------------------------------------
#include "LCD240X064.h"				
#include "Graphics.h"		
#include "DCF77_Auswertung.h"			
#include "Temperatursensor.h"
//-------------------------------------------------------------------------------------------------
//Functions
//-------------------------------------------------------------------------------------------------							 													
//=== Function: main ==============================================================================
///
/// @brief	Hauptprogramm
///
/// @param	void
///
/// @return	int
//=================================================================================================
int main(void)
{
	// Display initialisieren
	LcdInit();
	// Funkmodul initialisieren
	Dcf77Init();
	// Temperatursensor initialisieren
	MCP9808Init();
	// Variable zum aktivieren der Zeiterfasung 
	bool updateTime = true;
	
	// Dauerschleife
	while(1)
	{
		// Zeit wird ber das DCF77-Modul erfasst, wenn "updateTime" gesetzt ist
		// "updateTime" wird nach erfolgreicher Zeiterfassung automatisch wieder gelscht
		if (updateTime)
		{
			// Die if-Abfrage wird nur einmal durchlaufen, nmlich wenn das Empfangsmodul noch aus ist
			// Dieser Programmblock dient zum Einschalten des Empfngers 
			// In der ISR werden darufhin die Flanken am Ausgang des Emofngers gezhlt
			// und bei einer bestimmten Anzahl das Modul asl betriebsbereit deklariert ("dcf77ModuleReady")
			if (!(DCF77_MODULE_POWER_PIN & (1<<DCF77_MODULE_POWER_PIN_NUMBER)))
			{
				// Steuervariablen initialisieren
				dcf77LastSignalLevel = LOW_LEVEL;
				dcf77FirstEdgesCounter = 0;
				dcf77ResetInterruptCounter = true;
				dcf77SignalStartFound = false;
				dcf77BitCounter = 0;
				// Bitstrom-Puffer initialisieren
				Dcf77InitSignalBuffer();
				// DCF77-Modul einschalten
				Dcf77SwitchDcfModule(SWITCH_ON);
				// Cursor einschalten, um die aktuelle Statusmeldung zu markieren
				LcdSwitchCursor(SWITCH_ON);
				// Stausmeldung ausgeben (Auf Bereitschaft des Empfngermoduls warten)
				Dcf77WriteMessage(WAIT_FOR_MODULE);
			}
			// Wenn das Empfngermodul betriebsbereit ist: Statusmeldung ausgeben (Signalanfang wird gesucht)
			// Durch die Steuervariable "dcf77CurrentMessage" wird die Statusmeldung nur eimal ausgeben
			if ((dcf77FirstEdgesCounter == 3) 
				 && !dcf77SignalStartFound
				 && (dcf77CurrentMessage != MODULE_IS_READY_SEARCH_SIGNAL))
			{
				// Statusmeldung ausgeben 
				Dcf77WriteMessage(MODULE_IS_READY_SEARCH_SIGNAL);
			}
			// Wenn der Signalanfanng gefunden ist: Statusmeldung ausgeben (Signalanfang gefunden, Uhrzeit aktualisieren)
			// Durch die Steuervariable "dcf77CurrentMessage" wird die Statusmeldung nur eimal ausgeben
			if ((dcf77FirstEdgesCounter == 3) 
				 && dcf77SignalStartFound
				 && (dcf77CurrentMessage != MODULE_IS_READY_SIGNAL_FOUND_TIME_IS_UPDATING))
			{
				// Statusmeldung ausgeben
				Dcf77WriteMessage(MODULE_IS_READY_SIGNAL_FOUND_TIME_IS_UPDATING);
			}
			// Empfangenen Bitstrom auswerten
			if (dcf77BitCounter == NUMBER_OF_BITS)
			{
				// Bitstrom in Zeit und Datum konvertieren
				Dcf77CalculateTimeAndDate();
				// bertragung prfen
				// Nur wenn die bertragung fehlerfrei ist und die Daten konsistent (plausibel) sind,
				// gilt die Aktualisierung als erfolgreich und die Daten werden in die Uhrzeit kopiert
				if (Dcf77CheckTransmission()
					 && Dcf77CheckplausibilityOfData())
				{
					// Modul ausschalten
					Dcf77SwitchDcfModule(SWITCH_OFF);
					// Zeiterfassung beenden
					updateTime = false;
					// Errechnete Zeiten in Sytem-Uhrzeit kopieren
					systemTime.second = 0;
					systemTime.minute = copyTime.minute;
					systemTime.hour = copyTime.hour;
					systemTime.weekDay = copyTime.weekDay;
					systemTime.monthDay = copyTime.monthDay;
					systemTime.month = copyTime.month;
					systemTime.year = copyTime.year;
					// Cursor an eine nicht Sichtbare Stelle setzen
					// und ausschalten, da keine Statusmeldungen mehr vorliegen
					LcdSetCursor(CURSOR_PARK_POSITION_LINE,
											 CURSOR_PARK_POSITION_COLUMN);
					LcdSwitchCursor(SWITCH_OFF);
					// Bildschirm lschen
					LcdClear();
				}
				// Bei Fehlerhafter bertragung werden die Steuervariablen neu initialisiert
				// und die Signalerfassung beginnt von vorne 
				else
				{
					// Steuervariablen initialisieren
					dcf77LastSignalLevel = LOW_LEVEL;
					dcf77FirstEdgesCounter = 0;
					dcf77ResetInterruptCounter = true;
					dcf77SignalStartFound = false;
					dcf77BitCounter = 0;
					// Bitstrom-Puffer initialisieren
					Dcf77InitSignalBuffer();
					// Stausmeldung ausgeben (Auf Bereitschaft des Empfngermoduls warten)
					Dcf77WriteMessage(WAIT_FOR_MODULE);
				}
			}
		}
		// Uhrzeit, Datum und Temperatur jede Sekunde ausgeben
		// "timeIsSetet" wird jede Sekunde in der ISR wieder gelscht
		if (!timeIsSetet)
		{	
			if (!updateTime)
			{
				// Uhrzeit, Datum und Temperatur ausgeben
				// Uhrzeit und Datum sind in einem globalen struct gespeichert
				GraphicWriteDataOnLcd(MCP9808ReadAmbientTemperature());
			}
			// "timeIsSetet" setzten, damit die Daten nur ein mal pro Sekunde ausgegeben werden
			timeIsSetet = true;
		}
	}	
	return 0;
}